// OpenCV headers.
//#include "opencv/cv.h" // Sometimes cause strange errors in debug and C++ mode due to the redefinition of free()...
#include "opencv/cvwimage.h"
#include "opencv/cxcore.h"
#include "opencv/highgui.h"
// The following headers do not build in C mode.
#ifdef __cplusplus
#include "opencv/cvaux.h"
#include "opencv/cxmisc.h"
#include "opencv/ml.h"
#endif // __cplusplus
// min and max may be undefined by OpenCV 2.X.X so we need to redefine them here.
#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif // max
#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif // min

int main()
{
	// Open the webcam or a video file.
	//CvCapture* webcam0 = cvCreateCameraCapture(0);
	CvCapture* webcam0 = cvCreateFileCapture("ball.avi");

	int width = (int)cvGetCaptureProperty(webcam0, CV_CAP_PROP_FRAME_WIDTH);
	int height = (int)cvGetCaptureProperty(webcam0, CV_CAP_PROP_FRAME_HEIGHT);
	int nbTotalPixels = width*height;
	// Number of selected pixels in the picture.
	int nbSelectedPixels = 0;
	// Estimation of the center of the object in the picture.
	double obji = 0, objj = 0;
	// We only consider objects that covers that ratio of the picture (refers to the min size of the object on the picture).
	double objPixRatio = 1.0/256.0;
	// Colors of the object (obtained using the Color Picker and Edit Colors tools in Microsoft Paint).
	unsigned char rmin = 175, rmax = 255, gmin = 175, gmax = 255, bmin = 175, bmax = 255;
	
	int c = 0;
	IplImage* image = NULL;
	int i = 0, j = 0, index = 0;

	//CvFont font;
	//cvInitFont(&font, CV_FONT_HERSHEY_PLAIN, 1.0f, 1.0f);

	cvNamedWindow("Detection");

	for (;;)
	{
		// Wait up to 1 ms for the user to press a key and return the key code.
		c = cvWaitKey(1);

		// Stop if ESC key was pressed.
		if ((char)c == 27) break;

		// Initializations...
		nbSelectedPixels = 0;
		obji = 0;
		objj = 0;

		// Get an image from the webcam or video.
		image = cvQueryFrame(webcam0);

		// Wait for the user to press a key and stop if no more images.
		if (image == NULL)
		{
			printf("Unable to get an image.\n");
			c = cvWaitKey(0);
			break;
		}

		// Convert image->imageData from char* to unsigned char* to work with color values in 0..255.
		unsigned char* data = reinterpret_cast<unsigned char*>(image->imageData);

		for (i = 0; i < image->height; i++)
		{
			for (j = 0; j < image->width; j++)
			{
				index = 3*(j+image->width*i);
				double b = data[0+index];
				double g = data[1+index];
				double r = data[2+index];
				// Select the pixels with the right color.
				if (
					(r >= rmin)&&(r <= rmax)&&
					(g >= gmin)&&(g <= gmax)&&
					(b >= bmin)&&(b <= bmax)
					)
				{
					nbSelectedPixels++;
					// Prepare the computation of the mean of selected pixels.
					obji += i;
					objj += j;
					// Selected pixels are displayed in red.
					data[0+index] = 0;
					data[1+index] = 0;
					data[2+index] = 255;
				}
			}
		}

		if (nbSelectedPixels > nbTotalPixels*objPixRatio)
		{
			// Compute the mean of selected pixels.
			obji = obji/(double)nbSelectedPixels;
			objj = objj/(double)nbSelectedPixels;

			//cvPutText(image, "Object detected", cvPoint(10,20), &font, CV_RGB(0,255,0));
			cvRectangle(image, cvPoint((int)objj-50,(int)obji-50), cvPoint((int)objj+50,(int)obji+50), CV_RGB(0,255,0));
		}

		// Show the image in the window.
		cvShowImage("Detection", image);
	}

	cvDestroyWindow("Detection");

	cvReleaseCapture(&webcam0);

	return 0;
}
